package com.muyu.cloud.pay.controller;

import com.muyu.cloud.pay.domain.OrderPayCustomer;
import com.muyu.cloud.pay.domain.req.CustomerListReq;
import com.muyu.cloud.pay.domain.req.OrderCustomerAddReq;
import com.muyu.cloud.pay.domain.req.OrderCustomerUpdReq;
import com.muyu.cloud.pay.domain.resp.CustomerResp;
import com.muyu.cloud.pay.service.OrderPayCustomerService;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.common.system.domain.SysRole;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author：zhangchengzhi
 * @Package：com.muyu.cloud.pay.controller
 * @Project：cloud-pay
 * @name：OrderPayCustomerController
 * @Date：2024/7/30 21:24
 */
@Log4j2
@RestController
@RequestMapping("/customer")
@RequiredArgsConstructor
@Tag(name = "客户控制层", description = "进行客户管理,查看等相关操作")
public class OrderPayCustomerController {

    /**
     * 客户业务层
     */
    private final OrderPayCustomerService orderPayCustomerService;

    /**
     * 查询所有的客户
     *
     * @param customerListReq 客户列表请求参数
     * @return 客户列表
     */
    @RequestMapping(path = "/list", method = RequestMethod.POST)
    @Operation(summary = "查看客户", description = "根据客户的名称,编码,是否开启等可以进行客户的筛选")
    public Result<List<CustomerResp>> selectList(@Validated @RequestBody CustomerListReq customerListReq) {

        return Result.success(orderPayCustomerService.selectList(customerListReq));
    }

    ;

    /**
     * 获取所有客户的
     *
     * @return 客户集合
     */


    @GetMapping("/all")
    @Operation(summary = "获取未接入的客户", description = "调用nacosAPI获取所有的微服务名称,作为支付中台的客户")
    @Schema(description = "获取未接入的客户", defaultValue = "[\"客户1\",\"客户2\"]", type = "List")
    public Result<List<String>> getCustomerAllList() {

        return Result.success(
                orderPayCustomerService.getCustomerAllList()
        );//客户业务层
    }

    /**
     * 添加客户
     *
     * @param orderCustomerAddReq 客户信息
     * @return 添加结果
     */
    @PostMapping
    @Operation(summary = "客户信息添加", description = "添加支付平台客户信息,添加成功后才可以使用支付类的产品")
    public Result<String> save(@Validated @RequestBody OrderCustomerAddReq orderCustomerAddReq) {

        orderPayCustomerService.save(OrderPayCustomer.addBuild(orderCustomerAddReq));



        return Result.success();

    }


    /**
     * 修改客户
     *
     * @param orderCustomerId     修改客户请求信息
     * @param orderCustomerUpdReq
     * @return 修改结果
     */


    @PutMapping("/{orderCustomerId}")
    @Operation(summary = "客户信息修改", description = "通过ID修改客户信息")
    public Result<String> update(@Schema(title = "客户ID", type = "Long", defaultValue = "1", description = "修改客户信息需要依据的唯一条件")
                                 @PathVariable("orderCustomerId") Long orderCustomerId,
                                 @RequestBody @Validated OrderCustomerUpdReq orderCustomerUpdReq) {


        orderPayCustomerService.updateById(OrderPayCustomer.updBuild(orderCustomerUpdReq, () -> orderCustomerId));


        return Result.success(null, "操作成功");


    }

    /**
     * 删除客户
     *
     * @param orderCustomerId 删除客户请求信息
     * @return 删除结果
     */

    @DeleteMapping("/{orderCustomerId}")
    @Operation(summary = "客户信息删除", description = "通过ID删除客户信息,七天内有支付支付记录的客户,不可进行删除")
    public Result<String> delete(@PathVariable("orderCustomerId") Long orderCustomerId) {
        orderPayCustomerService.removeById(orderCustomerId);
        return Result.success(null, "操作成功");
    }

    /**
     * 通过ID获取客户
     *
     * @param orderCustomerId ID
     * @return 客户信息
     */

    @GetMapping("/{orderCustomerId}")
    @Operation(summary = "通过ID获取客户信息", description = "通过ID获取客户信息")
    public Result<OrderPayCustomer> findById(@PathVariable("orderCustomerId") Long orderCustomerId) {
        log.info("");
        return Result.success(orderPayCustomerService.getById(orderCustomerId), "操作成功");
    }


    /**
     * 通过ID禁用客户
     *
     * @param orderCustomerId ID
     * @return 禁用结果
     */

    @GetMapping("/disable/{orderCustomerId}")
    @Operation(summary = "通过ID禁用客户", description = "通过ID禁用客户,禁用之后禁止调用支付相关接口")
    public Result<OrderPayCustomer> disable(@PathVariable("orderCustomerId") Long orderCustomerId) {
        this.orderPayCustomerService.disable(orderCustomerId);
        return Result.success(null, "操作成功");
    }


    /**
     * 通过ID启动客户
     *
     * @param orderCustomerId ID
     * @return 启动结果
     */

    @GetMapping("/enable/{orderCustomerId}")
    @Operation(summary = "通过ID启动客户", description = "通过ID启用被禁止的客户,启用后可以调用支付相关的接口")
    public Result<OrderPayCustomer> enable(@PathVariable("orderCustomerId") Long orderCustomerId) {
        this.orderPayCustomerService.enable(orderCustomerId);
        return Result.success(null, "操作成功");
    }


    @Log(title = "导出支付", businessType = BusinessType.EXPORT)
    @RequiresPermissions("pay:customer:export")
    @PostMapping("/export")
    public void export (HttpServletResponse response,CustomerListReq customerListReq) {
     log.info("进行导出操作");
        List<CustomerResp> customerRespList = orderPayCustomerService.selectList(customerListReq);
        ExcelUtil<CustomerResp> util = new ExcelUtil<CustomerResp>(CustomerResp.class);
        util.exportExcel(response, customerRespList, "客户支付数据");
    }




}
